#ifndef Spieler_H
#define Spieler_H

#include <KSGNetwork.h>
#include <Model2D.h>
#include <TextFeld.h>
#include <Animation.h>
#include "..\Kamera\Kamera.h"
#include "../Team/Team.h"
#include "../Karte/Strukturen.h"
#include "DeadPlayer.h"

class Team;
class Laser;
class Karte;
class SpielerGUI;

enum TastaturStatus
{
	T_GAS,
	T_ROT_L,
	T_ROT_R,
	T_FEUER,
	T_MAX
};

class Spieler
{
	friend class SpielerGUI;
private:
    int accountId;
    int sNum;
    bool tastatur[ T_MAX ];
    Textur2D *textur;
    Model2D *ship;
    Team *team;
    KSGClient::InformationServerClient *info;
    Text *name;
    Vertex stL;
    Vertex kL;
    Vertex stR;
    Vertex kR;
    Vertex stM;
    Vertex kM;
    double akkuLeistungStart;
    double beschleunigungStart;
    double wendigkeitStart;
    struct SpielerDataSave
    {
        float rSpeed;
        int farbe;
        bool amLeben;
        double schussAbk;
        double energieAbk;
        double repAbk;
        double reinkAbk;
        int maxReinkAbk;
        int spielZeit;
        Punkt startPos;
        Vec2< double > pos;
        Vec2< double > speed;
        double rotation;
        double beschleunigung;
        double energie;
        double stabilitt;
        double reparatur;
        double laserIntensitt;
        double laserEffizienz;
        double akkuLeistung;
        double maxEnergie;
        double maxStabilitt;
        double laserTempo;
        double netzwerk;
        double wendigkeit;
        double antriebEffizienz;
        double zeitAmLeben;
        double zeitTod;
        double schadenBekommen;
        double schadenGemacht;
        double treibstoffVerbraucht;
        int schsse;
        int treffer;
        int punkte;
        int kills;
        int tode;
    };
    SpielerDataSave last;
    SpielerDataSave current;
	int ref;
	int skillPunkte;
    Textur2D *flammenM;
    Textur2D *flammenL;
    Textur2D *flammenR;
    TextFeld *nText;
    DeadPlayer *deadPlayer;

    double calculateLaserCost();

public:
	// Konstruktor
	Spieler( KSGClient::InformationServerClient *zInfoK, Schrift *zSchrift, SpielerStr *zStr );
	// Destruktor
	~Spieler();
	// nicht constant
	void setModelData( Model2DData *data, Animation2DData *zFStart, Animation2DData *zFBurn );
	void setAccountId( int accId );
	void setTeam( Team *team );
	void setSpielerFarbe( int fc, Bild *zTextur );
	void setGeschwindigkeit( double xSpeed, double ySpeed );
	void setPosition( Vec2< double > p );
	void setTastataturStatus( TastaturStatus ts, bool aktiv, int sZ, Karte *zMap );
	bool tick( int time, Karte *zMap );
	void render( Bild &zRObj );
	void renderLeben( Bild &zRObj );
	void renderMinimap( Bild &zRObj, Karte *zMap, int thisSpielerNummer );
	void setSkill( int art );
	void nimmSchaden( double intensitt );
    void calcDeadPlayerObject( Laser *zSchuss );
	void machSchaden( double intensitt );
	void addKill();
    DeadPlayer *sterben();
	void wiederbelebung();
	void resetToSaveTime( int t, Karte *zMap );
    void resetToGameTime( int t, Karte *zMap );
	void hatGeschossen();
	// constant
    Vec2< double > getSpeed() const;
	int getAccountId() const;
	Text *getName() const;
	Text *zName() const;
	bool hatSkill() const;
	bool istAmLeben() const;
	// Statistik Werte
	int getSpielerNummer() const;
	Team *zTeam() const;
	int getSpielerFarbe() const;
	int getSchadenBekommen() const;
	int getSchadenGemacht() const;
	int getTreibstoffVerbraucht() const;
	int getSchsse() const;
	int getTreffer() const;
	int getPunkte() const;
	int getKills() const;
	int getTode() const;
	int getZeitAmLeben() const;
	int getZeitTod() const;
	Punkt getPos() const;
	double getTeamMaxEnergieBonus() const;
	double getTeamMaxStabilittBonus() const;
	double getTeamReparaturBonus() const;
	double getTeamLaserIntensittBonus() const;
	double getTeamLaserEffizienzBonus() const;
	double getTeamAkkuLeistungBonus() const;
	double getTeamLaserTempoBonus() const;
	double getTeamBeschleunigungBonus() const;
	double getTeamWendigkeitBonus() const;
    double getTeamAntriebEffizienzBonus() const;
	// Reference Counting
	Spieler *getThis();
	Spieler *release();
};

#endif